/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * area.p
 *
 *
 * Storage Area Statistics monitoring.
 *
 *
 * To Do:
 *
 *	Area specific notes:
 *	  - bi??? clusters in use?
 *	  Fixed/Variable extent indicator
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	August 28, 2003
 *
 *
 * History:
 *
 *	Implemented Dmitri Levin's RPB solution
 *	September 22, 2003
 *
 */

{lib/protop.i}

define output parameter dcDescription as character no-undo initial "StorageAreas".

/*** moved to lib/vstlib.p cache
 ***
define temp-table tt_area no-undo
 ...
 ***
 ***/

/* initialize
 *
 */

define variable numAreas as integer no-undo.

procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_area:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "SAName",     "x(28)" ).
      altFormat( "80", b, "totGB",      ">>>>>9.999" ).
      altFormat( "80", b, "pctAlloc",   ">>>>>>9%" ).
      altFormat( "80", b, "pctLastX",   ">>>>9%" ).
      altFormat( "80", b, "clstrsz",    ">>9" ).
      altFormat( "80", b, "hasVar",     "Yes/No" ).
      altFormat( "80", b, "xnote",      "x(6)" ).
      altFormat( "80", b, "areaMaxPct", ">>>9.999%" ).
    end.

  run initAreaInfo.

  for each tt_area:
    numAreas = numAreas + 1.
  end.

  return.

end.

/* update
 *
 */

procedure mon-update:

  define input parameter argList as character no-undo.

  run getAreaVarInfo.

  publish "dba_trend".

  publish "resizeBrowse" ( "StorageAreas", numAreas ).

  add2ds( temp-table tt_area:default-buffer-handle ).

  return.

end.

return.
