/* lib/osinfo.i
 *
 * minimalist example:
 *
 *      define stream inStrm.		/* be wary of possible duplicates */
 *
 *	{lib/osinfo.i}
 *
 *	run osInfo.
 *
 *	for each tt_osInfo:
 *	  display tt_osInfo with side-labels 1 column.
 *	end.
 *
 * used by:
 *   dc/osinfo.p
 *   lib/install.p
 *   rp2017/environment.p
 *
 */

define temp-table tt_OSInfo no-undo

  field hostName     as character label "Host"      format "x(30)"
  field osName       as character label "OS"        format "x(100)"
  field osVersion    as character label "osVersion" format "x(30)"
  field keVersion    as character label "keVersion" format "x(30)"
  field osUpTime     as character label "Up Time"   format "x(30)"

  field maxFileSz    as character label "Max File Size"   format "x(16)"
  field maxOpenFl    as character label "Max Open Files"  format "x(16)"
  field maxProcMem   as character label "Max Proc Memory" format "x(16)"
  field maxProcs     as character label "Max Processes"   format "x(16)"

  field maxStack     as character label "Max Stack"       format "x(16)"
  field maxDataSeg   as character label "Max Data Seg"    format "x(16)"

  field SEMMNS       as character initial "n/a" format "x(16)"
  field SEMMNI       as character initial "n/a" format "x(16)"
  field SEMMSL       as character initial "n/a" format "x(16)"
  field SEMOPM       as character initial "n/a" format "x(16)"
  field SHMMAX       as character initial "n/a" format "x(16)"
  field SHMMNI       as character initial "n/a" format "x(16)"
  field SHMALL       as character initial "n/a" format "x(16)"
  field osPageSize   as integer

  field vmSwappiness as integer		/* echo 10 > /proc/sys/vm/swappiness		*/
  field vmDirtyRatio as integer		/* echo 15 > /proc/sys/vm/dirty_ratio		*/
  field vmDirtyBkRatio as integer	/* echo 3 > /proc/sys/vm/dirty_background_ratio	*/

  field clkCPU       as character format "x(16)"
  field numCPU       as character format "x(16)"	/* legacy	*/

  /* lscpu */

  field modelCPU       as character format "x(30)"
  field cpuArch        as character format "x(16)"
  field byteOrder      as character format "x(16)"
  field hypervisor     as character format "x(16)"
  field powerMgmt      as character format "x(16)"

  field numSockets     as integer   format ">>>>>9"
  field coresPerSocket as integer   format ">>>>>9"
  field threadsPerCore as integer   format ">>>>>9"
  field cpuCores       as integer   format ">>>>>9"		/* sockets * corePerSock		*/
  field logCPUs        as integer   format ">>>>>9"		/* sockets * corePerSock * threads	*/
  field activeCPUs     as integer   format ">>>>>9"		/* need to parse			*/

  field numaNodes      as integer   format ">>>>>9"

  field cpu_bogomips   as decimal   format ">>>>>>>>9.99"
  field cpu_clock      as decimal   format ">>>>>>>>9.99"

  /******/

  field numaStatus   as character format "x(16)"
  field vmStatus     as character format "x(16)"

  field memTotal     as character format "x(15)"
  field totalMem     as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memBufs      as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memCached    as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memShared    as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memActive    as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memInAct     as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memAvailable as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memFree      as decimal format ">>>>>>>>>>>>>>>>>>>>9"
  field memCommtdAS  as decimal format ">>>>>>>>>>>>>>>>>>>>9"

  index hostName-idx is unique primary hostName
.

define variable linuxPatch  as logical     no-undo initial yes.
define variable winBootTime as character   no-undo.
define variable winBootDTZ  as datetime-tz no-undo.


function ulimitValue returns character ( input inLine as character ):
  return trim( substring( inLine, r-index( inLine, ")" ) + 1 )).
end.


procedure getAIX:

  define variable inLine as character no-undo extent 16.
  define variable xLine  as character no-undo.

  find tt_OSInfo no-error.

  assign
    byteOrder  = "Big Endian"
    osPageSize = 4096
    numSockets = ?
    coresPerSocket = ?
  .

  input stream inStrm through value( "prtconf" ).
  repeat:

     xLine = "".
     import stream inStrm unformatted xLine.

     if       xLine begins "CPU Type"              then  cpuArch   = trim( entry( 2, xLine, ":" )).			/* CPU Type: 64-bit				*/
      else if xLine begins "Processor Type"        then  modelCPU  = entry( 2, trim( entry( 2, xLine, ":" )), "_" ).	/* Processor Type: PowerPC_POWER9		*/
      else if xLine begins "Number of Processors"  then  cpuCores  = decimal( trim( entry( 2, xLine, ":" ))).		/* Number Of Processors: 22			*/
      else if xLine begins "Processor Clock Speed" then  cpu_clock = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )).	/* Processor Clock Speed: 2850 MHz		*/

  end.
  input stream inStrm close.

  cpu_clock = cpu_clock / 1024.

  input stream inStrm through value( "lparstat -i" ).
  repeat:

     xLine = "".
     import stream inStrm unformatted xLine.

     if       xLine begins "Online Virtual CPUs"   then  logCPUs    = decimal( trim( entry( 2, xLine, ":" ))).		/* Online Virtual CPUs      : 22		*/
      else if xLine begins "Online Memory"         then  totalMem   = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )).		/* Online Memory            : 491520 MB		*/
      else if xLine begins "Type"                  then  hypervisor = trim( entry( 2, xLine, ":" )).			/* Type                     : Dedicated-SMT-4	*/
      else if xLine begins "Mode"                  then  hypervisor = hypervisor + entry( 2, xLine, ":" ).		/* Mode                     : Capped		*/

  end.
  input stream inStrm close.

  if index( hypervisor, "smt-" ) = 0 then
    threadsPerCore = 1.
   else
    threadsPerCore = integer( entry( 1, substring( hypervisor, index( hypervisor, "smt-" ) + 4 ), " " )) no-error.

  if cpuCores = logCPUs and threadsPerCore > 1 then
    cpuCores = logCPUs / threadsPerCore.

  input stream inStrm through value( "vmstat -v" ).
  repeat:

     xLine = "".
     import stream inStrm unformatted xLine.
    
     if       index( xLine, "free pages"     ) > 0 then  memfree    = decimal( substring( xLine, 1, 21 )).		/*     558520 free pages			*/
      else if index( xLine, "file pages"     ) > 0 then  membufs    = decimal( substring( xLine, 1, 21 )).		/*   74162695 file pages			*/
      else if index( xLine, "lruable pages"  ) > 0 then  memCached  = decimal( substring( xLine, 1, 21 )).		/*  122131280 lruable pages			*/

  end.
  input stream inStrm close.

  /* convert pages to GB
   */

  assign
    memfree   = ( memfree   * osPageSize ) / ( 1024 * 1024 * 1024 )
    membufs   = ( membufs   * osPageSize ) / ( 1024 * 1024 * 1024 )
    memCached = ( memCached * osPageSize ) / ( 1024 * 1024 * 1024 )
    totalMem  = totalMem / 1024						/* source is in MB not pages	*/
  .

  memTotal = trim( string( totalMem, ">>>>>>>>9.99" )) + " GB" no-error.

  input stream inStrm through value( "lssrad -va" ).									/*	REF1   SRAD        MEM      CPU		*/
  import stream inStrm unformatted xLine.										/*	0					*/
  repeat:														/*	          0   238513.31      0-43	*/
															/*	1					*/
     xLine = "".													/*	          1   238660.88      44-87	*/
     import stream inStrm unformatted xLine.
     if xLine <> "" then numaNodes = numaNodes + 1.

  end.

  numaNodes = numaNodes / 2.

  return.

end.

procedure getSunOS:

  define variable inLine as character no-undo extent 16.
  define variable xLine  as character no-undo.

  find tt_OSInfo no-error.

  /* $ uname -X
   * System = SunOS
   * Node = traxlive
   * Release = 5.10
   * KernelID = Generic_142900-03
   * Machine = sun4u
   * BusType = <unknown>
   * Serial = <unknown>
   * Users = <unknown>
   * OEM# = 0
   * Origin# = 1
   * NumCPU = 48
   */

  input stream inStrm through value( "uname -X" ).
  repeat:

     xLine = "".
     import stream inStrm unformatted xLine.

     if       index( xLine, "NumCPU" )   > 0 then numCPU    = trim( entry( 2, xLine, "=" )).
      else if index( xLine, "Release" )  > 0 then osVersion = trim( entry( 2, xLine, "=" )).
      else if index( xLine, "KernelID" ) > 0 then keVersion = trim( entry( 2, xLine, "=" )).

  end.
  input stream inStrm close.

  /* $ nslookup traxlive
   * Server:         10.1.8.21
   * Address:        10.1.8.21#53
   * 
   * Name:   traxlive.intcomex.com
   * Address: 10.1.10.178
   */

  /* $ /usr/sbin/prtconf | more
   * System Configuration:  Sun Microsystems  sun4u
   * Memory size: 131072 Megabytes
   * System Peripherals (Software Nodes):
   * 
   * SUNW,SPARC-Enterprise
   * ...
   */

  xLine = "".
  input stream inStrm through value( '/usr/sbin/prtconf | grep "Memory size:"' ).		/* reported in Megabytes...	*/
  import stream inStrm unformatted xLine.
  memTotal = trim( entry( 3, xLine, " " )).
  input stream inStrm close.

  memTotal = trim( string( decimal( memTotal ) / 1024, ">>>>>>>>9.99" )) + " GB" no-error.
  totalMem = decimal( entry( 1, memTotal, " " )).

  /*                            Old             Old       New         New
   *  Resource control          tunable         default   Max value   default
   *  ----------------------    -------------   -------   ---------   ----------
   *  process.max-sem-ops       seminfo_semopm  10        INT_MAX     512
   *  process.max-sem-nsems     seminfo_semmsl  25        SHRT_MAX    512
   *  project.max-sem-ids       seminfo_semmni  10        2**24       128
   *
   *  project.max-shm-memory    shminfo_shmmax  0x800000  UINT64_MAX  1/4 physmem
   *  project.max-shm-ids       shminfo_shmmni  100       2**24       128
   *
   *  $ prctl -n process.max-sem-ops $$
   *  process: 21592: -bash
   *  NAME    PRIVILEGE       VALUE    FLAG   ACTION                       RECIPIENT
   *  process.max-sem-ops
   *          privileged        512       -   deny                                 -
   *          system          2.15G     max   deny                                 -
   *
   *  $ prctl -n process.max-sem-ops $PPID | grep privileged
   *  semopm         privileged        512       -   deny 
   *
   *  $ prctl -n process.max-sem-ops -P $PPID | grep privileged
   *  process.max-sem-ops privileged 512 - deny -
   *
   */

  inLine = "".
  input stream inStrm through value( 'prctl -n process.max-sem-ops -P $PPID | grep privileged' ).
  import stream inStrm inLine.
  if inLine[2] <> "failed" then SEMOPM = trim( inLine[3] ).
  input stream inStrm close.

  inLine = "".
  input stream inStrm through value( 'prctl -n process.max-sem-nsems -P $PPID | grep privileged' ).
  import stream inStrm inLine.
  if inLine[2] <> "failed" then SEMMSL = trim( inLine[3] ).
  input stream inStrm close.

  inLine = "".
  input stream inStrm through value( 'prctl -n process.max-sem-ids $PPID | grep privileged' ).
  import stream inStrm inLine.
  if inLine[2] <> "failed" then SEMMNI = trim( inLine[3] ).
  input stream inStrm close.

  inLine = "".
  input stream inStrm through value( 'prctl -n process.max-shm-memory $PPID | grep privileged' ).
  import stream inStrm inLine.
  if inLine[2] <> "failed" then SHMMAX = trim( inLine[3] ).
  input stream inStrm close.

  inLine = "".
  input stream inStrm through value( 'prctl -n process.maxx-shm-ids $PPID | grep privileged' ).
  import stream inStrm inLine.
  if inLine[2] <> "failed" then SHMMNI = trim( inLine[3] ).
  input stream inStrm close.

  osPageSize = 4096.

  /* $ psrinfo -v | grep "operates at" | head -1
   *  The sparcv9 processor operates at 2530 MHz,
   */

  inLine = "".
  input stream inStrm through value( 'psrinfo -v | grep "operates at" | head -1' ).
  import stream inStrm inLine.
  if inLine[4] = "operates" then
    assign
      modelCPU = inLine[2]
      clkCPU   = inLine[6] + " " + inline[7]
    .
  input stream inStrm close.

  return.

end.

procedure getHPUX:

  define variable inLine as character no-undo extent 16.
  define variable xLine  as character no-undo.

  find tt_OSInfo no-error.

/* "machinfo"...
 * 
 * TXB448@cmp18697> machinfo
 * CPU info:
 *    Intel(R)  Itanium(R)  Processor 9350 (1.73 GHz, 18 MB)
 *    4 cores, 8 logical processors per socket
 *    4.79 GT/s QPI, CPU version E0
 *           Active processor count:
 *           2 sockets
 *           4 cores (1 to 3 per socket)
 *           4 logical processors
 *           LCPU attribute is disabled
 * 
 * Memory: 40953 MB (39.99 GB)
 * 
 * Firmware info:
 *    Firmware revision:  06.30.00.04
 *    FP SWA driver revision: 1.18
 *    IPMI is supported on this system.
 *    BMC firmware revision: 6.1e
 * 
 * Platform info:
 *    Model:                  "ia64 hp Integrity Virtual Partition"
 *    Machine ID number:      6ddae3a6-bb76-11e5-8802-ec9a746a811a
 *    Machine serial number:  VM01602000
 * 
 * OS info:
 *    Nodename:  cmp18697
 *    Release:   HP-UX B.11.31
 *    Version:   U (unlimited-user license)
 *    Machine:   ia64
 *    ID Number: 1843061670
 *    vmunix _release_version:
 * @(#) $Revision: vmunix:    B.11.31_LR FLAVOR=perf
 * 
 */

  input stream inStrm through value( '/usr/contrib/bin/machinfo' ).
  repeat:

    xLine = "".
    import stream inStrm unformatted xLine.

    if xLine begins "CPU Info:" then
      do:

        import stream inStrm unformatted xLine.

        modelCPU = trim( xLine ).
	modelCPU = replace( modelCPU, "(R)", "" ).
        modelCPU = replace( modelCPU, "Intel", "" ).
        modelCPU = replace( modelCPU, "Processor", "" ).

        cpu_clock = decimal( trim( entry( 1, entry( 2, modelCPU, "(" ), " " ))) no-error.
        clkCPU = trim( string( cpu_clock, ">>>>>>9.99" )).

        modelCPU = trim( entry( 1, modelCPU, "(" )).
        modelCPU = replace( modelCPU, "   ", " " ).
        modelCPU = replace( modelCPU,  "  ", " " ).
        modelCPU = trim( modelCPU ).

        import stream inStrm unformatted xLine.

        coresPerSocket = integer( trim( entry( 1, trim( xLine ), " " ))) no-error.
        logCPUs  = integer( trim( entry( 3, trim( xLine ), " " ))) no-error.		/* temporary usage of logCPU, this is logCPUs per socket	*/
        threadsPerCore = logCPUs / coresPerSocket.

      end.

    if trim( xLine ) begins "Active processor count:" then
      do:

        import stream inStrm unformatted xLine.
        numSockets  = integer( trim( entry( 1, trim( xLine ), " " ))) no-error.

        import stream inStrm unformatted xLine.
        cpuCores    = integer( trim( entry( 1, trim( xLine ), " " ))) no-error.

        import stream inStrm unformatted xLine.
        logCPUs     = integer( trim( entry( 1, trim( xLine ), " " ))) no-error.					/* actual logCPUs						*/

      end.

    if xLine begins "Memory:" then memTotal = entry( 1, entry( 2, xLine, "(" ), ")" ).				/* Memory: 40953 MB (39.99 GB)	*/

    if trim( xLine ) begins "Machine:" then cpuArch    = trim( entry( 2, xLine, ":" )).				/*    Machine:   ia64		*/
    if trim( xLine ) begins "Model:"   then osVersion  = trim( trim( entry( 2, xLine, ":" )), '"' ).		/*   Model:                  "ia64 hp Integrity Virtual Partition"	*/

  end.
  input stream inStrm close.

  threadsPerCore = logCPUs / cpuCores.

  if       index( osVersion, "Virtual" ) > 0 then hypervisor = substring( osVersion, index( osVersion, "Virtual" )).
   else if index( osVersion, "Shared"  ) > 0 then hypervisor = substring( osVersion, index( osVersion, "Shared" )).
   else hyperVisor = "unknown".

  assign
    byteOrder  = "Big Endian"
/*  osPageSize = 4096		*/
  .

  /* numaNodes =		*/
  /* cpu_bogomips =		*/

  /*    $ getconf PAGE_SIZE
   *    4096
   */

  input stream inStrm through value( 'getconf PAGE_SIZE' ).
  import stream inStrm unformatted xLine.
  osPageSize = integer( xLine ) no-error.
  input stream inStrm close.

  /*    $ swapinfo -Mm
   *                     Mb          Mb           Mb   PCT
   *    TYPE          AVAIL        USED         FREE  USED
   *    memory        38954       14777        24177   38%
   */

  input stream inStrm through value( 'swapinfo -Mm' ).
  import stream inStrm unformatted xLine.
  import stream inStrm unformatted xLine.
  import stream inStrm unformatted xLine.
  memFree = decimal( trim( substring( xLine, 32, 13 ))) / 1024 no-error.
  /* memActive = decimal( trim( substring( xLine, 20, 11 ))) / 1024 no-error.		*/	/* "Active" should refer to Linux style cache, not general memory	*/
  input stream inStrm close.

/* In HP/UX v 11, the command to display kernel parameters is kctune and we can grep to see the semaphore settings.
 *
 * $ /usr/sbin/kconfig -v
 * Configuration last_install
 * Title         Created by last OS install
 * Created       Tue Dec 14 12:29:49 2010 by root
 *               by saving the running configuration
 * Modified      Tue Dec 14 12:29:50 2010 by root
 * Kernel Path   /stand/last_install/vmunix
 * 
 * $ /usr/sbin/kctune -c last_install | grep sem
 *
 * sema                 1
 * semaem               16384
 * semmap               (SEMMNI+2)
 * semmni               200
 * semmns               800
 * semmnu               30
 * semume               10
 * semvmx               32767
 * 
 * $ /usr/sbin/kctune | grep -i shm
 * core_addshmem_read                          0  Default      Immed
 * core_addshmem_write                         0  Default      Immed
 * shmmax                             1073741824  Default      Immed
 * shmmni                                    400  Default      Immed
 * shmseg                                    300  Default      Immed
 * $ /usr/sbin/kctune | grep -i sem
 * semaem                                  16384  Default      Immed
 * semmni                                   2048  Default      Immed
 * semmns                                   4096  Default      Immed
 * semmnu                                    256  Default      Immed
 * semmsl                                   2048  Default      Immed
 * semume                                    100  Default      Immed
 * semvmx                                  32767  Default      Immed
 * 
 */

  input stream inStrm through value( '/usr/sbin/kctune' ).
  repeat:

    xLine = "".
    import stream inStrm unformatted xLine.

    if       xLine begins "semmns " then semmns = entry( 1, trim( substring( xLine, 8 )), " " ).
     else if xLine begins "semmni " then semmni = entry( 1, trim( substring( xLine, 8 )), " " ).
/*   else if xLine begins "semmnu " then semmnu = entry( 1, trim( substring( xLine, 8 )), " " ).		*/
/*   else if xLine begins "semmsl " then	semmsl = entry( 1, trim( substring( xLine, 8 )), " " ).		*/
/*   else if xLine begins "semopm " then semopm = entry( 1, trim( substring( xLine, 8 )), " " ).		*/

     else if xLine begins "shmmax " then shmmax = entry( 1, trim( substring( xLine, 8 )), " " ).
     else if xLine begins "shmmni " then shmmni = entry( 1, trim( substring( xLine, 8 )), " " ).
     else if xLine begins "shmall " then shmall = entry( 1, trim( substring( xLine, 8 )), " " ).

     else if xLine begins "base_pagesize " then osPageSize = integer( entry( 1, trim( substring( xLine, 15 )), " " )) * 1024 no-error.
     else if xLine begins "maxfiles "      then maxOpenFl  = entry( 1, trim( substring( xLine, 15 )), " " ).
     else if xLine begins "nproc "         then maxProcs   = entry( 1, trim( substring( xLine, 15 )), " " ).

  end.
  input stream inStrm close.

  return.

end.

procedure getWIN32:

  define variable inLine as character   no-undo extent 16.
  define variable xLine  as character   no-undo.

  define variable i      as integer     no-undo.

  find tt_OSInfo no-error.

  xLine = "".
  input stream inStrm through value( 'wmic os get caption' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted osName.
  input stream inStrm close.

  osName = replace( osName, "(R)", "" ).
  osName = replace( osName, "Microsoft", "" ).
  osName = trim( osName ).

  xLine = "".
  input stream inStrm through value( 'wmic os get version' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted osVersion.
  input stream inStrm close.

  xLine = "".
  input stream inStrm through value( 'wmic cpu get name' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted modelCPU.
  input stream inStrm close.
  modelCPU = replace( modelCPU, "(R)", "" ).
  modelCPU = replace( modelCPU, "(TM)", "" ).
  i = index( modelCPU, "CPU" ).
  if i > 0 then modelCPU = substring( modelCPU, 1, i - 2 ).
  i = index( modelCPU, "processor" ).
  if i > 0 then modelCPU = substring( modelCPU, 1, i - 2 ).

  xLine = "".
  input stream inStrm through value( 'wmic cpu get architecture' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted cpuArch.
  input stream inStrm close.

  case cpuArch:
    when "0" then cpuArch = "x86".
    when "1" then cpuArch = "MIPS".
    when "2" then cpuArch = "Alpha".
    when "3" then cpuArch = "PowerPC".
    when "6" then cpuArch = "ia64".
    when "9" then cpuArch = "x86_64".
    otherwise cpuArch = "unknown".
  end.

  byteOrder = "Little Endian".
  osPageSize = 4096.

  xLine = "".
  input stream inStrm through value( 'wmic computersystem get hypervisorpresent' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted hypervisor.
  input stream inStrm close.
  
  if trim( hypervisor ) <> "true" then
    hypervisor = "none".
   else
    do:
      xLine = "".
      input stream inStrm through value( 'wmic computersystem get model' ).
      import stream inStrm unformatted xline.
      import stream inStrm unformatted hypervisor.
      input stream inStrm close.
    end.

  xLine = "".
  input stream inStrm through value( 'wmic cpu get currentclockspeed' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted clkCPU.
  input stream inStrm close.

  /* clkCPU = trim( string( ( decimal( clkCPU ) / 1000.0 ), ">>>>>>9.99" )) + " Ghz". */

  cpu_clock = decimal( trim( clkCPU )) / 1024.

  assign
    cpuCores       = 0
    logCPUs        = 0
    numSockets     = 0
    coresPerSocket = 0
  .

  input stream inStrm through value( 'wmic cpu get socketdesignation,numberofenabledcore,numberoflogicalprocessors /format:list' ).
  repeat:
    import stream inStrm unformatted xLine.
    if xline begins "numberofenabledcore=" then
      do:
        cpuCores = cpuCores + integer( entry( 2, xLine, "=" )) no-error.
        coresPerSocket = max( coresPerSocket, integer( entry( 2, xLine, "=" ))) no-error.
      end.
     else if xline begins "numberoflogicalprocessors=" then logCPUs    = logCPUs  + integer( entry( 2, xLine, "=" )) no-error.
     else if xline begins "socketdesignation="         then numSockets = numSockets + 1.
  end.
  input stream inStrm close.

  if cpuCores = 0 then
    do:
      input stream inStrm through value( 'wmic cpu get socketdesignation,numberoflogicalprocessors /format:list' ).
      repeat:
        import stream inStrm unformatted xLine.
        if       xline begins "numberoflogicalprocessors=" then logCPUs    = logCPUs  + integer( entry( 2, xLine, "=" )) no-error.
         else if xline begins "socketdesignation="         then numSockets = numSockets + 1.
      end.
      input stream inStrm close.
    end.

  if numSockets = 0 then
    do:
      xLine = "".
      input stream inStrm through value( 'wmic computersystem get numberofprocessors' ).
      import stream inStrm unformatted xline.
      import stream inStrm unformatted numSockets no-error.
      input stream inStrm close.
    end.

  if cpuCores = 0 then
    do:
      xLine = "".
      input stream inStrm through value( 'wmic cpu get numberofcores' ).
      import stream inStrm unformatted xline.
      import stream inStrm unformatted cpuCores no-error.
      input stream inStrm close.
    end.

  if logCPUs = 0 then
    do:
      xLine = "".
      input stream inStrm through value( 'wmic cpu get numberoflogicalprocessors' ).
      import stream inStrm unformatted xline.
      import stream inStrm unformatted logCPUs no-error.
      input stream inStrm close.
    end.

  if cpuCores = 0 and numSockets > 0 then cpuCores = numSockets.	/* server2003 is missing some stats	*/
  if logCPUs  = 0 and numSockets > 0 then logCPUs  = numSockets.

  if coresPerSocket = 0 then
    coresPerSocket = cpuCores / numSockets no-error.

  if logCPUs > 0 then
    threadsPerCore = cpuCores / logCPUs no-error.


  xLine = "".
  input stream inStrm through value( 'wmic computersystem get totalphysicalmemory' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted memTotal.
  input stream inStrm close.
  memTotal = trim( string( ( decimal( memTotal ) / ( 1024 * 1024 * 1024 )), ">>>>>>>9.99" )) + " GB".

  totalMem = decimal( entry( 1, memTotal, " " )) no-error.

  xLine = "".
  input stream inStrm through value( 'wmic os get freephysicalmemory' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted memFree.
  input stream inStrm close.
  memFree = decimal( memFree ) / ( 1024 * 1024 ) no-error.	/* free memory is in KB not bytes like memTotal?  Yeesh!	*/

  xLine = "".
  input stream inStrm through value( 'wmic os get lastbootuptime' ).
  import stream inStrm unformatted xline.
  import stream inStrm unformatted winBootTime.
  input stream inStrm close.

  winBootDTZ = datetime(
    integer( substring( winBootTime,  5, 2 )),		/* month	*/
    integer( substring( winBootTime,  7, 2 )),		/* day		*/
    integer( substring( winBootTime,  1, 4 )),		/* year		*/
    integer( substring( winBootTime,  9, 2 )),		/* hours	*/
    integer( substring( winBootTime, 11, 2 )),		/* minutes	*/
    integer( substring( winBootTime, 13, 2 )),		/* seconds	*/
    integer( integer( substring( winBootTime, 16, 6 )) / 1000 )		/* milliseconds	*/
  ) no-error.
/***
    integer( substring( winBootTime, 22 ))		/* TZ		*/
  ).
 ***/

  osUpTime =
    string( abs ( interval( winBootDTZ, now, "days" )), ">>>>>>" ) + " " +
    string( abs ( interval( winBootDTZ, now, "seconds" )), "hh:mm:ss" )
  no-error.

  osUpTime = trim( osUpTime ).

  return.

end.

procedure getLinux:

  define variable inLine as character no-undo extent 16.
  define variable xLine  as character no-undo.

  find tt_OSInfo no-error.

  if osName = "" then						/* this data won't change without a reboot so don't keep checking		*/
    do:

      if search( "/etc/os-release" ) <> ? then			/* this should be the standard - all Linuxes with systemd should have it	*/
        do:
          input stream inStrm from value( "/etc/os-release" ).
          repeat:
          import stream inStrm unformatted xLine.
            if       xline begins "name=" then osName    = trim( entry( 2, xLine, "="), '" ' ).
             else if xline begins "version=" then osVersion = trim( entry( 2, xLine, "="), '" ' ).
          end.
          input stream inStrm close.
        end.
       else if search( "/etc/SuSE-release" ) <> ? then            /* SuSE */
        /* sample:
        SUSE Linux Enterprise Server 10 (x86_64)
        VERSION = 10
        PATCHLEVEL = 2
        */
        do:
          input stream inStrm from value( "/etc/SuSE-release" ).
          import stream inStrm unformatted xLine.
          osName = xline. /* substring( xLine, 1, 30 ). */
          xLine = "".
          import stream inStrm unformatted xLine.
          osVersion = entry( 3, xLine, " " ) + ".".
          xLine = "".
          import stream inStrm unformatted xLine.
          osVersion = osVersion + entry( 3, xLine, " " ).
          input stream inStrm close.
        end.
       else if search( "/etc/centos-release" ) <> ? then          /* CentOS */
        /* sample:
        CentOS release 6.5 (Final)
        */
        do:
          input stream inStrm from value( "/etc/centos-release" ).
          import stream inStrm unformatted xLine.
          input stream inStrm close.
          osName = xLine.
          osVersion = entry( 2, substring( xLine, index( xLine, "release" ) ), " " ).
        end.
       else if search( "/etc/redhat-release" ) <> ? then          /* RH (or a clone?) */
        /* sample:
        Red Hat Enterprise Linux Server release 6.3 (Santiago)
        */
        do:
          input stream inStrm from value( "/etc/redhat-release" ).
          import stream inStrm unformatted xLine.
          input stream inStrm close.
          osName = xLine.
          osVersion = entry( 2, substring( xLine, index( xLine, "release" ) ), " " ).
        end.
       else                                                       /* some other Linux */
        do:
          input stream inStrm through value( 'uname' ).
          import stream inStrm osName.
          input stream inStrm close.
          osVersion = "Unknown".
        end.

      input stream inStrm through value( "getconf PAGESIZE" ).
      import stream inStrm osPageSize.   /* in bytes */
      input stream inStrm close.

    end.

/*
 *  input stream inStrm from value( "/proc/cpuinfo" ).
 *  repeat:
 *    xLine = "".
 *    import stream inStrm unformatted xLine.
 *    if       xLine begins "model name"   then modelCPU     = entry( 2, xLine, ":" ).
 *     else if xLine begins "cpu MHz"      then clkCPU       = trim( string( decimal( trim( entry( 2, xLine, ":" ))) / 1024, ">>>>>9.99" )).
 *     else if xLine begins "cache size"   then cpu_cache    = trim( entry( 2, xLine, ":" )).
 *     else if xLine begins "cpu cores"    then cpu_cores    = trim( entry( 2, xLine, ":" )).
 *     else if xLine begins "siblings"     then cpu_siblings = trim( entry( 2, xLine, ":" )).
 *     else if xLine begins "bogomips"     then cpu_bogomips = trim( entry( 2, xLine, ":" )).
 *     else if xLine begins "power management" then leave.
 *  end.
 *  input stream inStrm close.
 */

  input stream inStrm through value( 'lscpu' ).		/* this could probably be extracted from /proc/cpuinfo and/or /sys/devices/system/cpu/cpu*	*/
  repeat:

    xLine = "".
    import stream inStrm unformatted xLine.

    if       xLine begins "Model name:"         then modelCPU       = trim( entry( 2, xLine, ":" )).
     else if xLine begins "Architecture:"       then cpuArch        = trim( entry( 2, xLine, ":" )).
     else if xLine begins "Byte Order:"         then byteOrder      = trim( entry( 2, xLine, ":" )).
     else if xLine begins "Hypervisor"          then hypervisor     = trim( entry( 2, xLine, ":" )).

     else if xLine begins "CPU(s):"             then logCPUs        = integer( trim( entry( 2, xLine, ":" ))).
     else if xLine begins "Socket(s):"          then numSockets     = integer( trim( entry( 2, xLine, ":" ))).		/* different versions of lscpu		*/
     else if xLine begins "CPU Socket(s):"      then numSockets     = integer( trim( entry( 2, xLine, ":" ))).		/* label cpu socket(s) differently	*/
     else if xLine begins "Core(s)"             then coresPerSocket = integer( trim( entry( 2, xLine, ":" ))).
     else if xLine begins "Thread(s) per core"  then threadsPerCore = integer( trim( entry( 2, xLine, ":" ))).
     else if xLine begins "NUMA node(s):"       then numaNodes      = integer( trim( entry( 2, xLine, ":" ))).

     else if xLine begins "BogoMIPS:"           then cpu_bogomips   = decimal( trim( entry( 2, xLine, ":" ))).
     else if xLine begins "CPU Mhz:"            then cpu_clock      = decimal( trim( entry( 2, xLine, ":" ))).

  end.
  input stream inStrm close.

  if hypervisor = "" then hypervisor = "None Found".

  numaStatus = "Undetermined".

  xLine = "".
  input stream inStrm through value( "numactl -H" ).
  repeat:
    import stream inStrm unformatted xLine.
    if xLine begins "available" then numaStatus = entry( 1, trim( entry( 2, xline, ":" )), "(" ).
  end.
  input stream inStrm close.


  /* no shelling out, simply reading /proc values. IOW input FROM rather that input THROUGH
   */

  input stream inStrm from value( "/proc/sys/kernel/sem" ).
  import stream inStrm SEMMSL SEMMNS SEMOPM SEMMNI.
  input stream inStrm close.

  input stream inStrm from value( "/proc/sys/kernel/shmmax" ).
  import stream inStrm SHMMAX.
  input stream inStrm close.

  SHMMAX = string( decimal( SHMMAX ) / ( 1024 * 1024 * 1024 ), ">>>>>>>>>>>>>>>>>>>>9" ) + " GB".

  input stream inStrm from value( "/proc/sys/kernel/shmmni" ).
  import stream inStrm SHMMNI.
  input stream inStrm close.

  input stream inStrm from value( "/proc/sys/kernel/shmall" ).
  import stream inStrm SHMALL.
  input stream inStrm close.

  SHMALL = string(( decimal( SHMALL ) / ( 1024 * 1024 * 1024 )) * osPageSize, ">>>>>>>>>>>>>>>>>>>>9" ) + " GB".

  file-info:file-name = "/proc/sys/vm/swappiness".
  if file-info:full-pathname <> ? then
    do:
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm vmSwappiness no-error.
      input stream inStrm close.
    end.

  file-info:file-name = "/proc/sys/vm/dirty_ratio".
  if file-info:full-pathname <> ? then
    do:
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm vmDirtyRatio no-error.
      input stream inStrm close.
    end.

  file-info:file-name = "/proc/sys/vm/dirty_background_ratio".
  if file-info:full-pathname <> ? then
    do:
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm vmDirtyBkRatio no-error.
      input stream inStrm close.
    end.

  file-info:file-name = "/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor".
  if file-info:full-pathname <> ? then
    do:
      input stream inStrm from value( file-info:full-pathname ).
      import stream inStrm powermgmt no-error.
      input stream inStrm close.
    end.
  if powermgmt = "" or powermgmt = ? then powermgmt = "Unknown".


  if byteOrder = "" and cpuArch begins "x86" then
    byteOrder = "Little Endian".

  if byteOrder = "" then					/* still blank?				*/
    byteOrder = "Unknown".

  cpuCores = logCPUs.						/* default, in case threadsPerCore = 0	*/
  cpuCores = logCPUs / threadsPerCore no-error.			/* real cores				*/

  cpu_clock = cpu_clock / 1000.					/* adjust from Mhz to Ghz		*/

  if modelCPU = "" then
    do:
      input stream inStrm from value( "/proc/cpuinfo" ).
      repeat:
        xLine = "".
        import stream inStrm unformatted xLine.
        if        xLine begins "model name"   then modelCPU       = entry( 2, xLine, ":" ).
          else if xLine begins "bogomips"     then cpu_bogomips   = decimal( trim( entry( 2, xLine, ":" ))).
/*        else if xLine begins "cpu cores"    then cpuCores       = integer( trim( entry( 2, xLine, ":" ))).	*/
/*        else if xLine begins "siblings"     then coresPerSocket = integer( trim( entry( 2, xLine, ":" ))).	*/
      end.
      input stream inStrm close.
    end.

  if modelCPU <> "" then
    do:
      assign
        modelCPU = entry( 1, modelCPU, "@" )
        modelCPU = replace( modelCPU, "(R)", "" )
        modelCPU = replace( modelCPU, "Intel", "" )
        modelCPU = replace( modelCPU, "CPU", "" )
      .
      do while index( modelCPU, "  " ) > 0:
        modelCPU = replace( modelCPU, "  ", " " ).
      end. 
    end.

  input stream inStrm from value( "/proc/meminfo" ).
  repeat:
    xLine = "".
    import stream inStrm unformatted xLine.
    if       xLine begins "MemTotal"   then memTotal  = trim( string( decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ), ">>>>>>>>>>>>9.99 GB" )).
  /*** this is more of a realtime dashboard -- nmon or top are much better for this - OTOH we sometimes need to alert on this stuff
   ***/
     else if xLine begins "MemAvailable"  then memAvailable = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "MemFree"       then memFree      = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Buffers"       then memBufs      = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Cached"        then memCached    = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Shmem"         then memShared    = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Active:"       then memActive    = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Inactive:"     then memInAct     = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
     else if xLine begins "Committed_AS:" then memCommtdAS  = decimal( entry( 1, trim( entry( 2, xLine, ":" )), " " )) / ( 1024 * 1024 ).
  /***
   ***/
  end.
  input stream inStrm close.

  totalMem = decimal( entry( 1, memTotal, " " )).

  return.

end.

procedure osInfo:

  define variable inLine   as character no-undo extent 16.
  define variable xLine    as character no-undo.
  define variable osFamily as character no-undo.

  find tt_OSInfo no-error.
  if not available( tt_osinfo ) then create tt_osinfo.

  input stream inStrm through value( "uname -a" ).
  import stream inStrm osFamily ^ keVersion.
  input stream inStrm close.

  hostName = getHostName().

  /*
   *  if opsys begins "WIN" then
   *    do:
   *      if session:batch = no then osFamily = "Please apply the Linux patch ;)".
   *      linuxPatch = yes.
   *    end.
   */

  if opsys = "unix" then
    do:

      input stream inStrm through value( "ulimit -a" ).		/* this should be portable across all unix implementations...	*/
								/* except, of course, for HP-UX :(				*/
      repeat:

        /* $ ulimit -a
         * time(seconds)        unlimited
         * file(blocks)         unlimited
         * data(kbytes)         3932160
         * stack(kbytes)        262140
         * memory(kbytes)       unlimited
         * coredump(blocks)     4194303
         */

        xLine = "".
        import stream inStrm unformatted xLine.

        if index( xLine, "-f)" ) > 0 or xLine begins "file(blocks)" then
          do:
            maxFileSz  = trim( substring( xLine, r-index( xLine, ")" ) + 1 )).											/* ulimit -f	*/
            if maxFileSz <> "unlimited" then 
              maxFileSz  = string( decimal( maxFileSz ) / ( 1024 * 1024 * 2 )) + " GB" no-error.						/* ulimit value is in "blocks"	*/
          end.
         else if index( xLine, "-n)" ) > 0 then maxOpenFl  = trim( substring( xLine, r-index( xLine, ")" ) + 1 )) no-error.					/* ulimit -n	*/
         else if index( xLine, "-u)" ) > 0 then maxProcs   = trim( substring( xLine, r-index( xLine, ")" ) + 1 )) no-error.					/* ulimit -u	*/
         else if index( xLine, "-s)" ) > 0 or xLine begins "stack(kbytes)"  then maxStack   = trim( substring( xLine, r-index( xLine, ")" ) + 1 )) no-error.	/* ulimit -s	*/
         else if index( xLine, "-d)" ) > 0 or xLine begins "data(kbytes)"   then maxDataSeg = trim( substring( xLine, r-index( xLine, ")" ) + 1 )) no-error.	/* ulimit -d	*/
         else if index( xLine, "-m)" ) > 0 or xLine begins "memory(kbytes)" then maxProcMem = trim( substring( xLine, r-index( xLine, ")" ) + 1 )) no-error.	/* ulimit -m	*/

      end.
      input stream inStrm close.

    end.

  if       osFamily = "Linux" then run getLinux.
   else if osFamily = "AIX"   then run getAIX.
   else if osFamily = "SunOS" then run getSunOS.
   else if osFamily = "HP-UX" then run getHPUX.
   else if opsys begins "WIN" then run getWIN32.		/* would ' = "win32"' be better?	*/

  return.

end.
